/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include "gwf_projopt.h"
#include <gwf_lib.h>

static void		gwf_projopt_class_init		(GwfProjOptClass	*klass);
static void		gwf_projopt_init		(GwfProjOpt		*projopt);
static void		gwf_projopt_destroy		(GtkObject		*object);

static void 		gwf_projopt_update 		(GwfProjOpt *projopt);
static void		gwf_projopt_add_act_plug	(GwfProjData *data,gpointer user_data);

/******************************************/
/* Project Signal connect & disconnecting */
/******************************************/
static void		gwf_projopt_disconnect_project 	(GwfProject *project,GwfProjOpt *projopt);
static void		gwf_projopt_connect_project 	(GwfProject *project,GwfProjOpt *projopt);

/************************************/
/* Project & Project list Callbacks */
/************************************/
static void	curproj_changed_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void 	project_added_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void	author_changed_cb		(GwfProject *project, gchar *auhthor,gpointer user_data);
static void	email_changed_cb		(GwfProject *project, gchar *email,gpointer user_data);
static void	cfile_changed_cb		(GwfProject *project, gchar *c_file,gpointer user_data);
static void	hfile_changed_cb		(GwfProject *project, gchar *h_file,gpointer user_data);
static void	license_changed_cb		(GwfProject *project, gchar *license,gpointer user_data);
static void	backup_changed_cb		(GwfProject *project, gboolean backup,gpointer user_data);
static void	example_changed_cb		(GwfProject *project, gboolean example,gpointer user_data);
static void	plugin_active_toggled_cb	(GwfProject *project, gchar *name, gpointer user_data);

/*****************/
/* GUI Callbacks */
/*****************/
static void		on_btnTogglePlugin_clicked	(GtkButton *button,gpointer user_data);
static gboolean		on_author_focus_out		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);
static gboolean		on_email_focus_out		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);
static gboolean		on_cfile_focus_out		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);
static gboolean		on_hfile_focus_out		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);
static gboolean		on_license_focus_out		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);
static void		on_backup_toggled		(GtkToggleButton *button, gpointer user_data);
static void             on_example_toggled		(GtkToggleButton *button, gpointer user_data);
static void		on_btnSourceFile_clicked	(GtkButton *button, gpointer user_data);

/****************/
/* GUI creation */
/****************/
static void		gwf_projopt_fill_licenses	(GwfProjOpt *projopt,GwfLicList *liclist);
static void		gwf_projopt_interface 		(GwfProjOpt *projopt);
static void		gwf_projopt_notebook_label	(GtkNotebook *nb,gint page,gchar *lab);
static GtkWidget*	gwf_projopt_notebook 		(GwfProjOpt *projopt);
static GtkWidget*	gwf_projopt_general_tab 	(GwfProjOpt *projopt);
static GtkWidget*	gwf_projopt_save_options_tab 	(GwfProjOpt *projopt);
static GtkWidget*	gwf_projopt_save_files_tab 	(GwfProjOpt *projopt);
static GtkWidget*	gwf_projopt_active_plugins_tab	(GwfProjOpt *projopt);
static void		gwf_projopt_add_license 	(GwfLicense *license,gpointer user_data);


guint
gwf_projopt_get_type (void)
{
  static guint gwf_projopt_type = 0;

  if (!gwf_projopt_type)
    {
      GtkTypeInfo gwf_projopt_info =
      {
	"GwfProjOpt",
	sizeof (GwfProjOpt),
	sizeof (GwfProjOptClass),
	(GtkClassInitFunc) gwf_projopt_class_init,
	(GtkObjectInitFunc) gwf_projopt_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_projopt_type = gtk_type_unique (gtk_vbox_get_type(), &gwf_projopt_info);
    }

  return gwf_projopt_type;
}

static void		
gwf_projopt_class_init (GwfProjOptClass *klass)
{
  ((GtkObjectClass *)klass)->destroy = gwf_projopt_destroy;
}

static void
gwf_projopt_init (GwfProjOpt *projopt)
{
  GtkStyle	*style;
  GdkColor	*color = NULL;

  gwf_projopt_interface (projopt);

  style = gtk_widget_get_style(GTK_WIDGET(projopt->act_plugs));
  color = &(style->base[GTK_STATE_NORMAL]);

  projopt->item_set = gwf_gdkpixmap_create_with_transp(GTK_WIDGET(projopt->act_plugs),item_set_xpm,color);
}

void
gwf_projopt_construct(GwfProjOpt *projopt,GwfProjlist *projlist,GwfLicList *liclist)
{
  projopt->projlist = projlist;
  gwf_projlist_project_foreach(projopt->projlist,(GwfProjlistForeachFunc)gwf_projopt_connect_project,
     			       (gpointer)projopt);
  gwf_projopt_update(projopt);

  gwf_projopt_fill_licenses (projopt,liclist);
  gtk_signal_connect (GTK_OBJECT (projlist), "curproj_changed",
                      GTK_SIGNAL_FUNC(curproj_changed_cb),
                      (gpointer)projopt);  
  gtk_signal_connect (GTK_OBJECT (projlist), "project_added",
                      GTK_SIGNAL_FUNC (project_added_cb),
                      (gpointer)projopt);
}

GtkWidget *
gwf_projopt_new(GwfProjlist *projlist,GwfLicList *liclist)
{
  GwfProjOpt		*projopt;
  
  projopt = GWF_PROJOPT(gtk_type_new(gwf_projopt_get_type()));
  gwf_projopt_construct(projopt,projlist,liclist);
  return GTK_WIDGET(projopt);
}

static void
gwf_projopt_destroy (GtkObject *object)
{
  gwf_projlist_project_foreach(GWF_PROJOPT(object)->projlist,(GwfProjlistForeachFunc)gwf_projopt_disconnect_project,
     			       (gpointer)object);
  gtk_signal_disconnect_by_data(GTK_OBJECT(GWF_PROJOPT(object)->projlist),(gpointer)object);
}

static void 
gwf_projopt_update (GwfProjOpt *projopt)
{
  GwfProject		*project;
  
  gtk_clist_clear(projopt->act_plugs);
  project = gwf_projlist_get_current(projopt->projlist);
  if (project)
  {
    gtk_entry_set_text(projopt->author,  NULL_TO_EMPTY(gwf_project_get_author(project)));
    gtk_entry_set_text(projopt->email,   NULL_TO_EMPTY(gwf_project_get_email(project)));
    gtk_entry_set_text(projopt->cfile,   NULL_TO_EMPTY(gwf_project_get_cfile(project)));
    gtk_entry_set_text(projopt->hfile,   NULL_TO_EMPTY(gwf_project_get_hfile(project)));
    gtk_entry_set_text(projopt->license, NULL_TO_EMPTY(gwf_project_get_license(project)));
    
    gtk_toggle_button_set_active(projopt->backup, gwf_project_get_backup(project));
    gtk_toggle_button_set_active(projopt->example,gwf_project_get_example(project));

    gwf_data_list_foreach(gwf_project_get_data_list(project),gwf_projopt_add_act_plug,(gpointer)projopt);
  }
  else
  {
    gtk_entry_set_text(projopt->author,"");
    gtk_entry_set_text(projopt->email,"");
    gtk_entry_set_text(projopt->cfile,"");
    gtk_entry_set_text(projopt->hfile,"");
    gtk_entry_set_text(projopt->license,"");
    gtk_toggle_button_set_active(projopt->backup,FALSE);
    gtk_toggle_button_set_active(projopt->example,FALSE);
  }
}

static void
gwf_projopt_add_act_plug (GwfProjData *data,gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  gint		row;
  gchar		*temp[2] = {NULL};

  temp[0] = "";
  temp[1] = data->name;

  row = gtk_clist_append(projopt->act_plugs,temp);

  if (gwf_proj_data_get_active(data))
    gtk_clist_set_pixtext(projopt->act_plugs,row,0,"",0,projopt->item_set,NULL);
  else
    gtk_clist_set_text(projopt->act_plugs,row,0,"");
}

/******************************************/
/* Project Signal connect & disconnecting */
/******************************************/

static void
gwf_projopt_disconnect_project (GwfProject *project,GwfProjOpt *projopt)
{
  gtk_signal_disconnect_by_data(GTK_OBJECT(project),(gpointer)projopt);
}

static void
gwf_projopt_connect_project (GwfProject *project,GwfProjOpt *projopt)
{
  gtk_signal_connect (GTK_OBJECT (project), "author_changed",
     		      GTK_SIGNAL_FUNC (author_changed_cb),
                      (gpointer)projopt);
  gtk_signal_connect (GTK_OBJECT (project), "email_changed",
     		      GTK_SIGNAL_FUNC (email_changed_cb),
                      (gpointer)projopt);
  gtk_signal_connect (GTK_OBJECT (project), "cfile_changed",
     		      GTK_SIGNAL_FUNC (cfile_changed_cb),
                      (gpointer)projopt);
  gtk_signal_connect (GTK_OBJECT (project), "hfile_changed",
     		      GTK_SIGNAL_FUNC (hfile_changed_cb),
                      (gpointer)projopt);
  gtk_signal_connect (GTK_OBJECT (project), "backup_changed",
     		      GTK_SIGNAL_FUNC (backup_changed_cb),
                      (gpointer)projopt);
  gtk_signal_connect (GTK_OBJECT (project), "example_changed",
     		      GTK_SIGNAL_FUNC (example_changed_cb),
                      (gpointer)projopt);
  gtk_signal_connect (GTK_OBJECT (project), "license_changed",
     		      GTK_SIGNAL_FUNC (license_changed_cb),
                      (gpointer)projopt);
  gtk_signal_connect (GTK_OBJECT (project), "plugin_active_toggled",
                      GTK_SIGNAL_FUNC (plugin_active_toggled_cb),
                      (gpointer)projopt);
}

/************************************/
/* Project & Project list Callbacks */
/************************************/

static void
curproj_changed_cb (GwfProjlist *projlist,gchar *name,gpointer user_data)
{
  gwf_projopt_update(GWF_PROJOPT(user_data));
}

static void
project_added_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  GwfProject	*project;

  project = gwf_projlist_get_project(projlist,name);

  gwf_projopt_connect_project (project,GWF_PROJOPT(user_data));
}

static void	
author_changed_cb		(GwfProject *project, gchar *author,gpointer user_data)
{
  if (!gwf_projlist_project_is_current(GWF_PROJOPT(user_data)->projlist,project))
    return;
    
  gtk_entry_set_text(GTK_ENTRY(GWF_PROJOPT(user_data)->author),NULL_TO_EMPTY(author));
}

static void	
email_changed_cb		(GwfProject *project, gchar *email,gpointer user_data)
{
  if (!gwf_projlist_project_is_current(GWF_PROJOPT(user_data)->projlist,project))
    return;
    
  gtk_entry_set_text(GTK_ENTRY(GWF_PROJOPT(user_data)->email),NULL_TO_EMPTY(email));
}

static void	
cfile_changed_cb		(GwfProject *project, gchar *c_file,gpointer user_data)
{
  if (!gwf_projlist_project_is_current(GWF_PROJOPT(user_data)->projlist,project))
    return;
    
  gtk_entry_set_text(GTK_ENTRY(GWF_PROJOPT(user_data)->cfile),NULL_TO_EMPTY(c_file));
}

static void	
hfile_changed_cb		(GwfProject *project, gchar *h_file,gpointer user_data)
{
  if (!gwf_projlist_project_is_current(GWF_PROJOPT(user_data)->projlist,project))
    return;
    
  gtk_entry_set_text(GTK_ENTRY(GWF_PROJOPT(user_data)->hfile),NULL_TO_EMPTY(h_file));
}

static void
backup_changed_cb		(GwfProject *project,gboolean backup,gpointer user_data)
{
  if (!gwf_projlist_project_is_current(GWF_PROJOPT(user_data)->projlist,project))
    return;
    
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(GWF_PROJOPT(user_data)->backup),backup);
}

static void
example_changed_cb		(GwfProject *project,gboolean example,gpointer user_data)
{
  if (!gwf_projlist_project_is_current(GWF_PROJOPT(user_data)->projlist,project))
    return;
  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(GWF_PROJOPT(user_data)->example),example);
}

static void
license_changed_cb		(GwfProject *project,gchar *license,gpointer user_data)
{
  if (!gwf_projlist_project_is_current(GWF_PROJOPT(user_data)->projlist,project))
    return;
    
  gtk_entry_set_text(GTK_ENTRY(GWF_PROJOPT(user_data)->license),NULL_TO_EMPTY(license));
}

static void
plugin_active_toggled_cb (GwfProject *project, gchar *name, gpointer user_data)
{
  GwfProjOpt		*projopt = user_data;
  gint			row;

  if (!gwf_projlist_project_is_current(projopt->projlist,project))
    return;
		
  row = gtk_clist_get_row_by_text(projopt->act_plugs,1,name);
  g_return_if_fail(row != -1);

  if (gwf_project_plugin_get_active(project,name))
    gtk_clist_set_pixtext(projopt->act_plugs,row,0,"",0,projopt->item_set,NULL);
  else
    gtk_clist_set_text(projopt->act_plugs,row,0,"");
}

/*****************/
/* GUI Callbacks */
/*****************/
static gboolean
on_author_focus_out(GtkWidget *widget,GdkEventFocus *event,gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  gchar		*tmp;
  GwfProject	*project;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return FALSE;

  tmp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
  gwf_project_set_author(project,NULL_TO_EMPTY(tmp));
  g_free(tmp);

  return FALSE;
}

static gboolean
on_email_focus_out(GtkWidget *widget,GdkEventFocus *event,gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  gchar		*tmp;
  GwfProject	*project;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return FALSE;

  tmp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
  gwf_project_set_email(project,NULL_TO_EMPTY(tmp));
  g_free(tmp);

  return FALSE;
}

static gboolean
on_cfile_focus_out(GtkWidget *widget,GdkEventFocus *event,gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  gchar		*tmp;
  GwfProject	*project;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return FALSE;

  tmp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
  gwf_project_set_cfile(project,NULL_TO_EMPTY(tmp));
  g_free(tmp);

  return FALSE;
}

static gboolean
on_hfile_focus_out(GtkWidget *widget,GdkEventFocus *event,gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  gchar		*tmp;
  GwfProject	*project;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return FALSE;

  tmp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
  gwf_project_set_hfile(project,NULL_TO_EMPTY(tmp));
  g_free(tmp);

  return FALSE;
}

static gboolean
on_license_focus_out(GtkWidget *widget,GdkEventFocus *event,gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  gchar		*tmp;
  GwfProject	*project;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return FALSE;

  tmp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
  gwf_project_set_license(project,NULL_TO_EMPTY(tmp));
  g_free(tmp);

  return FALSE;
}

static void
on_backup_toggled (GtkToggleButton *button, gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  GwfProject	*project;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return;

  gwf_project_set_backup(project,gtk_toggle_button_get_active(button));
}

static void
on_example_toggled (GtkToggleButton *button, gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  GwfProject	*project;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return;

  gwf_project_set_example(project,gtk_toggle_button_get_active(button));
}

static void
on_btnTogglePlugin_clicked		(GtkButton       *button,
   					 gpointer         user_data)
{
  GwfProjOpt	*projopt = user_data;
  GList		*tmp;
  gchar		*name;
  GwfProject	*project;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return;

  for (tmp = projopt->act_plugs->selection ; tmp ; tmp = tmp->next)
  {
    g_return_if_fail(gtk_clist_get_text(projopt->act_plugs,GPOINTER_TO_INT(tmp->data),1,&name));
    gwf_project_plugin_set_active(project,name,!gwf_project_plugin_get_active(project,name));
  }
}

static void
on_btnSourceFile_clicked (GtkButton *button, gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;
  GwfProject	*project;
  gchar		*widname,*tmp;

  project = gwf_projlist_get_current(projopt->projlist);
  if (!project)
    return;

  widname = g_strdup(gwf_project_get_widname(project));
  g_strdown(widname);

  gwf_project_set_cfile(project,widname);
  gwf_project_set_hfile(project,widname);

  g_free(widname);
}

/****************/
/* GUI creation */
/****************/

static void
gwf_projopt_fill_licenses (GwfProjOpt *projopt,GwfLicList *liclist)
{
  gwf_liclist_foreach(liclist,gwf_projopt_add_license,(gpointer)projopt);
  gtk_entry_set_text(projopt->license,liclist->std);
}

static void
gwf_projopt_add_license (GwfLicense *license,gpointer user_data)
{
  GwfProjOpt	*projopt = user_data;

  gtk_combo_entry_add_item(projopt->license,license->name);
}

static void
gwf_projopt_interface (GwfProjOpt *projopt)
{
  gtk_box_pack_start (GTK_BOX (projopt), gwf_projopt_notebook(projopt), TRUE, TRUE, 0);
}

static void
gwf_projopt_notebook_label(GtkNotebook *nb,gint page,gchar *lab)
{
  GtkWidget 	*label;
  
  label = gtk_label_new (lab);
  gtk_widget_show (label);
  gtk_notebook_set_tab_label (nb, gtk_notebook_get_nth_page (nb, page), label);
}

static GtkWidget*
gwf_projopt_notebook (GwfProjOpt *projopt)
{
  GtkWidget	*notebook;
  
  notebook = gtk_notebook_new ();
  gtk_widget_show (notebook);

  gtk_container_add (GTK_CONTAINER (notebook), gwf_projopt_general_tab(projopt));
  gwf_projopt_notebook_label(GTK_NOTEBOOK(notebook),0,"General :");

  gtk_container_add (GTK_CONTAINER (notebook), gwf_projopt_save_options_tab(projopt));
  gwf_projopt_notebook_label(GTK_NOTEBOOK(notebook),1,"Save Options :");

  gtk_container_add (GTK_CONTAINER (notebook), gwf_projopt_save_files_tab(projopt));
  gwf_projopt_notebook_label(GTK_NOTEBOOK(notebook),2,"Save Files :");

  gtk_container_add (GTK_CONTAINER (notebook), gwf_projopt_active_plugins_tab(projopt));
  gwf_projopt_notebook_label(GTK_NOTEBOOK(notebook),3,"Active Plugins :");
  
  return notebook;
}

static GtkWidget*
gwf_projopt_general_tab (GwfProjOpt *projopt)
{
  GtkWidget	*table;
  GtkWidget	*label;
  GtkWidget	*entry;

  table = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table);

  label = gtk_label_new ("Email :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  label = gtk_label_new ("Author :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  projopt->author = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  gtk_signal_connect(GTK_OBJECT(entry), "focus-out-event",
  		     GTK_SIGNAL_FUNC(on_author_focus_out),
  		     (gpointer)projopt);
  		
  entry = gtk_entry_new ();
  projopt->email = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  gtk_signal_connect(GTK_OBJECT(entry), "focus-out-event",
  		     GTK_SIGNAL_FUNC(on_email_focus_out),
  		     (gpointer)projopt);
  		

  return table;
}

static GtkWidget*
gwf_projopt_save_options_tab (GwfProjOpt *projopt)
{
  GtkWidget	*table;
  GtkWidget	*checkbutton;
  GtkWidget	*label;
  GtkWidget	*combo;
  
  table = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table);

  checkbutton = gtk_check_button_new_with_label ("Bacup source files");
  projopt->backup = GTK_TOGGLE_BUTTON(checkbutton);
  gtk_widget_show (checkbutton);
  gtk_table_attach (GTK_TABLE (table), checkbutton, 0, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  gtk_signal_connect(GTK_OBJECT(checkbutton), "toggled",
                     GTK_SIGNAL_FUNC(on_backup_toggled),
                     (gpointer)projopt);

  checkbutton = gtk_check_button_new_with_label ("Generate example program");
  projopt->example = GTK_TOGGLE_BUTTON(checkbutton);
  gtk_widget_show (checkbutton);
  gtk_table_attach (GTK_TABLE (table), checkbutton, 0, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  /* FIXME: Until example generating is ready*/
  gtk_widget_set_sensitive(checkbutton,FALSE);
  /* */

  gtk_signal_connect(GTK_OBJECT(checkbutton), "toggled",
                     GTK_SIGNAL_FUNC(on_example_toggled),
                     (gpointer)projopt);

  label = gtk_label_new("License type :");
  gtk_widget_show(label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  combo = gtk_combo_new();
  gtk_widget_show(combo);
  gtk_combo_set_value_in_list(GTK_COMBO(combo),TRUE,TRUE);
  projopt->license = GTK_ENTRY(GTK_COMBO(combo)->entry);
  gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "focus-out-event",
  		     GTK_SIGNAL_FUNC(on_license_focus_out),
  		     (gpointer)projopt);
  		
  return table;
}

static GtkWidget*
gwf_projopt_save_files_tab (GwfProjOpt *projopt)
{
  GtkWidget	*table;
  GtkWidget	*label;
  GtkWidget	*entry;
  GtkWidget	*button;

  table = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table);

  label = gtk_label_new ("Source C-file :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  projopt->cfile = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_signal_connect(GTK_OBJECT(entry), "focus-out-event",
  		     GTK_SIGNAL_FUNC(on_cfile_focus_out),
  		     (gpointer)projopt);
  		

  label = gtk_label_new ("Source H-file :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  projopt->hfile = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_signal_connect(GTK_OBJECT(entry), "focus-out-event",
  		     GTK_SIGNAL_FUNC(on_hfile_focus_out),
  		     (gpointer)projopt);
  		
  button = gtk_button_new_with_label("From widget name");
  gtk_widget_show(button);
  gtk_table_attach (GTK_TABLE (table), button, 1, 2, 2, 3,
                    (GtkAttachOptions) ( GTK_EXPAND ),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(on_btnSourceFile_clicked), (gpointer)projopt);
  		
  return table;
}

static GtkWidget*
gwf_projopt_active_plugins_tab(GwfProjOpt *projopt)
{
  GtkWidget	*vbox;
  GtkWidget	*button;
  GtkWidget	*clist;
  gchar		*titles[2] = {"Active","Plugin"};
  
  vbox = gtk_vbox_new(FALSE,0);
  gtk_widget_show(vbox);
  
  clist = gtk_clist_new_with_titles(2,titles);
  projopt->act_plugs = GTK_CLIST(clist);
  gtk_widget_show(clist);
  gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_MULTIPLE);
  gtk_box_pack_start(GTK_BOX(vbox),clist,TRUE,TRUE,5);
  
  button = gtk_button_new_with_label("Toggle Active");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,5);
  
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnTogglePlugin_clicked),
                      (gpointer)projopt);
  
  return vbox;
}
