/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef GWF_TOOLS_H
#define GWF_TOOLS_H

#ifdef __cplusplus
extern "C" {
#endif

#include <gtk/gtk.h>
#include "gwf_project.h"
#include "gwf_projlist.h"

/**************************************/
/* Standard string conversion macro's */
/**************************************/
#define NULL_TO_EMPTY(txt)		((txt == NULL) ? "" : txt)
#define EMPTY_TO_NULL(txt)		((strlen(g_strstrip(txt)) == 0) ? NULL : txt)

#define BOOL_TO_STR(val)		((val) ? "TRUE" : "FALSE")
#define STR_TO_BOOL(val)		((g_strcasecmp(val,"TRUE")==0) ? TRUE : FALSE)

/**********************/
/* GtkCList functions */
/**********************/
gint		gtk_clist_get_row_by_text	(GtkCList *clist,gint column,gchar *text);
gchar*		gtk_clist_get_sel_row_col_text	(GtkCList *clist,gint column);

/***********************/
/* GtkWindow functions */
/***********************/
void		gwf_window_show			(GtkWidget *window);
void		gwf_window_set_title		(GtkWindow *window,gchar *windowname,gchar *project);
void		gwf_window_set_title_current	(GtkWindow *window,gchar *windowname,GwfProjlist *projlist);
void		gwf_parent_child_windows_connect(GtkWindow *parent,GtkWindow *child);
void		gwf_window_hide_on_destroy	(GtkWindow *window);

/**********************/
/* GtkCTree functions */
/**********************/
GtkCTreeNode*	gtk_ctree_add_node		(GtkCTree *tree,GtkCTreeNode *prnt,gchar *name);
void		gtk_ctree_alter_node		(GtkCTree *tree,GtkCTreeNode *node,gchar *text);

void		gtk_ctree_freeze		(GtkCTree *tree);
void		gtk_ctree_thaw			(GtkCTree *tree);

GtkCTreeNode*	gtk_ctree_find_item		(GtkCTree *tree,GtkCTreeNode *parent,gchar *item);
GtkCTreeNode*	gtk_ctree_find_item_part	(GtkCTree *tree,GtkCTreeNode *parent,gchar *item);
GtkCTreeNode*	gtk_ctree_find_project		(GtkCTree *tree,GwfProject *project);
GtkCTreeNode*	gtk_ctree_find_project_item	(GtkCTree *tree,GwfProject *project,gchar *item);

gint		gtk_ctree_insert_text_value	(GtkCTree *tree,GtkCTreeNode *parent,gchar *item,gchar *value);
gint		gtk_ctree_insert_bool_value	(GtkCTree *tree,GtkCTreeNode *parent,gchar *item,gboolean value);
gint		gtk_ctree_insert_float_value	(GtkCTree *tree,GtkCTreeNode *parent,gchar *item,gfloat value);
gint		gtk_ctree_insert_int_value	(GtkCTree *tree,GtkCTreeNode *parent,gchar *item,glong value);
void		gtk_ctree_alter_text_value	(GtkCTree *tree,GtkCTreeNode *node,gchar *item,gchar *value);
void		gtk_ctree_alter_bool_value	(GtkCTree *tree,GtkCTreeNode *node,gchar *item,gboolean value);
void		gtk_ctree_alter_float_value	(GtkCTree *tree,GtkCTreeNode *node,gchar *item,gfloat value);
void    	gtk_ctree_alter_int_value	(GtkCTree *tree,GtkCTreeNode *node,gchar *item,glong value);

/************************/
/* Additional functions */
/************************/

void		gtk_combo_entry_add_item	(GtkEntry *entry,gchar *text);

gchar*		gtk_type_to_string		(GtkType type);
GtkType		string_to_gtk_type		(gchar *type);
GtkType		find_best_gtk_type		(gchar *gtktype,gchar *strtype);

GList*		gwf_open_dir			(gchar *dir);

#ifdef __cplusplus
}
#endif

#endif

